package com.widget.imagevideobanner.log;

import android.os.Environment;
import android.text.TextUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class BannerLogFileUtils {

    private static ExecutorService sExecutor;
    private static SimpleDateFormat sdf = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss");
    private static final String BANNER_LOG_DIR = Environment.getExternalStorageDirectory().getPath()
            + File.separator
            + "miyaterminal"
            + File.separator
            + "log"
            + File.separator
            + "bannererr.log";

    static {
        File logFiles = new File(BANNER_LOG_DIR);
        if (!logFiles.exists()) {
            logFiles.mkdirs();
        }
    }

    public static void writeLog(final String input) {
        if (sExecutor == null) {
            sExecutor = Executors.newSingleThreadExecutor();
        }
        sExecutor.execute(new Runnable() {
            @Override
            public void run() {
                writeFileLog(input);
            }
        });
    }

    private static void writeFileLog(String input) {
        if (TextUtils.isEmpty(input)) {
            return;
        }
        input = sdf.format(new Date()) + input + "\n";

        FileOutputStream fos = null;
        OutputStreamWriter writer = null;
        try {
            File logFile = new File(BANNER_LOG_DIR);
            if (!logFile.exists() && !logFile.createNewFile()) return;
            fos = new FileOutputStream(logFile, true);
            writer = new OutputStreamWriter(fos, Charset.forName("utf-8"));
            writer.write(input);
            writer.flush();
            writer.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (null != fos) {
                try {
                    fos.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != writer) {
                try {
                    writer.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }





}
