package com.widget.imagevideobanner.view;

import android.content.Context;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.view.PagerAdapter;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.widget.imagevideobanner.R;
import com.widget.imagevideobanner.bean.MediaBean;
import com.widget.imagevideobanner.player.IPlayTarget;

import java.util.ArrayList;
import java.util.List;

import static com.widget.imagevideobanner.bean.MediaBean.TYPE_IMAGE;
import static com.widget.imagevideobanner.bean.MediaBean.TYPE_VIDEO;

/**
 * 图片+视频
 * 混播控件
 */
public class ImageVideoBanner extends FrameLayout implements ViewPager.OnPageChangeListener, OnActionFinishListener {

    private static String TAG = "ImageVideoBanner";
    private ViewPager mViewPager;
    private List<View> mList = new ArrayList<>();
    private ViewsPagerAdapter mAdapter;
    private int curIndex;

    private ImageVideoBanner(@NonNull Context context) {
        super(context);
        initView();
    }

    public ImageVideoBanner(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        initView();
    }

    private void initView(){
        View view =  LayoutInflater.from(getContext()).inflate(R.layout.banner_imge_video, this, true);
        mViewPager = view.findViewById(R.id.view_pager);
        mAdapter = new ViewsPagerAdapter(mList);
        mViewPager.setAdapter(mAdapter);
        mViewPager.setOnPageChangeListener(this);
    }

    public void bindData(List<MediaBean> beans){
        if(beans == null){
            return;
        }
        mList.clear();
        for(MediaBean mediaBean : beans){
            if(!mediaBean.isNull()){
                if(mediaBean.getResType() == TYPE_IMAGE){
                    mList.add(new VpImageView(getContext(),mediaBean,this));
                }

                if(mediaBean.getResType() == TYPE_VIDEO){
                    mList.add(new VpVideoView(getContext(),mediaBean,this));
                }
            }
        }
        if(mList.size() > 0){
            mAdapter.notifyDataSetChanged();
        }
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        IPlayTarget iPlayTarget = (IPlayTarget)mList.get(curIndex);
        iPlayTarget.onActive();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        IPlayTarget iPlayTarget = (IPlayTarget)mList.get(curIndex);
        iPlayTarget.inActive();
    }

    @Override
    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {}

    @Override
    public void onPageSelected(int position) {
        IPlayTarget iPlayTarget = (IPlayTarget)mList.get(curIndex);
        iPlayTarget.onActive();
    }

    @Override
    public void next() {
        curIndex = curIndex+1;
        if (curIndex >= mList.size()) {
            curIndex = 0;
        }
        Log.e(TAG,"curIndex = " + curIndex);
        mViewPager.setCurrentItem(curIndex);
    }

    @Override
    public void onPageScrollStateChanged(int state) { }

    private static final class ViewsPagerAdapter extends PagerAdapter {

        private List<View> mList;

        public ViewsPagerAdapter(List<View> list) {
            this.mList = list;
        }

        @Override
        public int getCount() {
            return mList.size();
        }

        @Override
        public boolean isViewFromObject(@NonNull View view, @NonNull Object object) {
            return view == object;
        }

        @NonNull
        @Override
        public View instantiateItem(ViewGroup container, int position) {
            container.addView(mList.get(position));
            return mList.get(position);
        }
        @Override
        public void destroyItem(ViewGroup container, int position, @NonNull Object object) {
            container.removeView(mList.get(position));
        }

    }

}
