package com.widget.imagevideobanner.view;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.widget.FrameLayout;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.google.android.exoplayer2.ExoPlaybackException;
import com.google.android.exoplayer2.PlaybackParameters;
import com.google.android.exoplayer2.Player;
import com.google.android.exoplayer2.SimpleExoPlayer;
import com.google.android.exoplayer2.Timeline;
import com.google.android.exoplayer2.source.MediaSource;
import com.google.android.exoplayer2.source.TrackGroupArray;
import com.google.android.exoplayer2.trackselection.TrackSelectionArray;
import com.google.android.exoplayer2.ui.PlayerControlView;
import com.google.android.exoplayer2.ui.PlayerView;
import com.widget.imagevideobanner.R;
import com.widget.imagevideobanner.bean.MediaBean;
import com.widget.imagevideobanner.player.IPlayTarget;
import com.widget.imagevideobanner.player.PlayerManager;
import com.widget.imagevideobanner.utils.ThrowableUtils;

import java.math.BigDecimal;

import jp.wasabeef.glide.transformations.BlurTransformation;

public class VpVideoView extends FrameLayout implements IPlayTarget, PlayerControlView.VisibilityListener, Player.EventListener {

    private static final String TAG = "VpVideoView";
    private ImageView cover, blur;
    private String mVideoUrl;
    private int mWidthPx;
    private int mHeightPx;
    private View bufferView;
    private OnActionFinishListener mOnPageChangedListener;
    private MediaBean mediaBean;
    private boolean isVideoEnded = false;

    public VpVideoView(@NonNull Context context,MediaBean bean, final OnActionFinishListener onPageChangedListener) {
        super(context);
        LayoutInflater.from(context).inflate(R.layout.layout_player_view, this, true);
        //封面view
        cover = findViewById(R.id.cover);
        //高斯模糊背景图,防止出现两边留嘿
        blur = findViewById(R.id.blur_background);
        //缓冲转圈圈的view
        bufferView = findViewById(R.id.buffer_view);
        //播放完成监听
        mOnPageChangedListener = onPageChangedListener;
        this.mediaBean = bean;
        bindData(mediaBean.getWidth(),mediaBean.getHeight(),mediaBean.getCover(),mediaBean.getUrl());
    }

    public void bindData(int widthPx, int heightPx, String coverUrl, String videoUrl) {
        mVideoUrl = videoUrl;
        mWidthPx = widthPx;
        mHeightPx = heightPx;
        setCoverImageUrl(coverUrl,cover);
        setBlurImageUrl(coverUrl,blur);
    }

    private void setBlurImageUrl(String blurUrl,ImageView blur){
        Glide.with(getContext())
                .load(blurUrl)
                .transform(new BlurTransformation())
                .dontAnimate()
                .listener(new RequestListener<Drawable>() {
                    @Override
                    public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Drawable> target, boolean isFirstResource) {
                        Log.e(TAG, ThrowableUtils.getFullStackTrace(e));
                        return false;
                    }

                    @Override
                    public boolean onResourceReady(Drawable resource, Object model, Target<Drawable> target, DataSource dataSource, boolean isFirstResource) {
                        return false;
                    }
                }).into(blur);
    }


    //todo 缓存固定尺寸
    private void setCoverImageUrl(String coverUrl,ImageView cover){
        Glide.with(getContext())
                .load(coverUrl)
                .dontAnimate()
                .listener(new RequestListener<Drawable>() {
                    @Override
                    public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Drawable> target, boolean isFirstResource) {
                        Log.e(TAG, ThrowableUtils.getFullStackTrace(e));
                        return false;
                    }

                    @Override
                    public boolean onResourceReady(Drawable resource, Object model, Target<Drawable> target, DataSource dataSource, boolean isFirstResource) {
                        return false;
                    }
                }).into(cover);
    }


    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        Log.e(TAG, "onSizeChanged");
        reSize();
    }

    private void reSize(){
        //动态设置
        int mHeight = getHeight();
        int mWidth = getWidth();

        BigDecimal viewScale = new BigDecimal(mWidth+"")
                .divide(new BigDecimal(mHeight+""),2, BigDecimal.ROUND_HALF_UP);
        BigDecimal videoScale = new BigDecimal(mWidthPx+"")
                .divide(new BigDecimal(mHeightPx+""),2, BigDecimal.ROUND_HALF_UP);

        if(viewScale.compareTo(videoScale) >= 0){
            //按照高度缩放
            BigDecimal heightScale = new BigDecimal(mHeight+"")
                    .divide(new BigDecimal(mHeightPx+""),2, BigDecimal.ROUND_HALF_UP);
            mHeightPx = mHeight;
            mWidthPx =  new BigDecimal(mWidthPx+"").multiply(heightScale).intValue();
        }else{
            //按照宽度缩放
            BigDecimal widthScale = new BigDecimal(mWidth+"")
                    .divide(new BigDecimal(mWidthPx+""),2, BigDecimal.ROUND_HALF_UP);
            mWidthPx = mWidth;
            mHeightPx =  new BigDecimal(mHeightPx+"").multiply(widthScale).intValue();
        }

        FrameLayout.LayoutParams coverParams = (LayoutParams) cover.getLayoutParams();
        coverParams.width = mWidthPx;
        coverParams.height = mHeightPx;
        coverParams.gravity = Gravity.CENTER;
        cover.setLayoutParams(coverParams);
        Log.e(TAG, "onSizeChanged"+mWidthPx+"  "+mHeightPx);
    }

    @Override
    public void onTimelineChanged(Timeline timeline, Object manifest, int reason) {

    }

    @Override
    public void onTracksChanged(TrackGroupArray trackGroups, TrackSelectionArray trackSelections) {

    }

    @Override
    public void onLoadingChanged(boolean isLoading) {

    }

    @Override
    public void onPlayerStateChanged(boolean playWhenReady, int playbackState) {
        Log.e(TAG, "onPlayerStateChanged playbackState"+playbackState);
        SimpleExoPlayer exoPlayer = PlayerManager.getInstance().exoPlayer;
        if (playbackState == Player.STATE_READY && exoPlayer.getBufferedPosition() != 0 && playWhenReady) {
            cover.setVisibility(GONE);
            bufferView.setVisibility(GONE);
        } else if (playbackState == Player.STATE_BUFFERING) {
            cover.setVisibility(VISIBLE);
            bufferView.setVisibility(VISIBLE);
        }

        //执行2次 bugfxied
        if(playbackState == Player.STATE_ENDED){
            if(!isVideoEnded){
                isVideoEnded = true;
                playNextVideo();
            }
        }
    }

    private void playNextVideo(){
        Log.e(TAG, "playNextVideo");
        inActive();
        if(null != mOnPageChangedListener){
            mOnPageChangedListener.next();
        }
    }

    @Override
    public void onRepeatModeChanged(int repeatMode) {

    }

    @Override
    public void onShuffleModeEnabledChanged(boolean shuffleModeEnabled) {

    }

    @Override
    public void onPlayerError(ExoPlaybackException error) {
        Log.e(TAG, ThrowableUtils.getFullStackTrace(error));
        playNextVideo();
    }

    @Override
    public void onPositionDiscontinuity(int reason) {

    }

    @Override
    public void onPlaybackParametersChanged(PlaybackParameters playbackParameters) {

    }

    @Override
    public void onSeekProcessed() {

    }

    @Override
    public void onVisibilityChange(int visibility) {
    }



    @Override
    public void onActive() {
        isVideoEnded = false;
        bufferView.setVisibility(GONE);
        cover.setVisibility(VISIBLE);
        //todo 这个地方能否得到具体尺寸
        //视频播放,或恢复播放
        //通过该View所在页面的mCategory(比如首页列表tab_all,沙发tab的tab_video,标签帖子聚合的tag_feed) 字段，
        //取出管理该页面的Exoplayer播放器，ExoplayerView播放View,控制器对象PageListPlay
        PlayerManager playerManager = PlayerManager.getInstance();
        SimpleExoPlayer exoPlayer = playerManager.exoPlayer;
        PlayerView playerView = playerManager.playerView;

        //此处我们需要主动调用一次 switchPlayerView，把播放器Exoplayer和展示视频画面的View ExoplayerView相关联
        //为什么呢？因为在列表页点击视频Item跳转到视频详情页的时候，详情页会复用列表页的播放器Exoplayer，然后和新创建的展示视频画面的View ExoplayerView相关联，达到视频无缝续播的效果
        //如果 我们再次返回列表页，则需要再次把播放器和ExoplayerView相关联
        playerManager.switchPlayerView(playerView, true);
        ViewParent parent = playerView.getParent();
        if (parent != this) {
            //把展示视频画面的View添加到ItemView的容器上
            if (parent != null) {
                ((ViewGroup) parent).removeView(playerView);
            }
            //视频尺寸参数共用cover
            ViewGroup.LayoutParams coverParams = cover.getLayoutParams();
            this.addView(playerView, 1, coverParams);
        }
        //如果是同一个视频资源,则不需要从重新创建mediaSource。
        //但需要onPlayerStateChanged 否则不会触发onPlayerStateChanged()
        if (TextUtils.equals(playerManager.playUrl, mVideoUrl)) {
            onPlayerStateChanged(true, Player.STATE_READY);
        } else {
            MediaSource mediaSource = playerManager.createMediaSource(mVideoUrl);
            exoPlayer.prepare(mediaSource);
            //TODO 如果只有一个视频 模式改为循环播放
            exoPlayer.setRepeatMode(Player.REPEAT_MODE_OFF);
            playerManager.playUrl = mVideoUrl;
        }
        exoPlayer.addListener(this);
        exoPlayer.setPlayWhenReady(true);
    }

    @Override
    public void inActive() {
        Log.e(TAG, "inActive");
        PlayerManager playerManager = PlayerManager.getInstance();
        playerManager.exoPlayer.setPlayWhenReady(false);
        playerManager.exoPlayer.removeListener(this);
        View child = getChildAt(1);
        if(child instanceof PlayerView){
            PlayerView playerView = (PlayerView)child;
            playerView.setPlayer(null);
            playerManager.playerView.setPlayer(null);
            this.removeViewInLayout(child);
        }
    }

}
